@@ -18,6 +18,7 @@
 
 import argparse
 import atexit
+import base64
 import collections
 import codecs
 import functools
@@ -65,7 +66,7 @@ signal.signal(signal.SIGINT, sigint_handler)
 
 # Constants
 
-_VERSION_ = '3.1'
+_VERSION_ = '3.2'
 
 COLORMAP = {k: '\x1b[%sm' % v for k, v in {
     'a': '30', 'b': '31', 'c': '32', 'd': '33',
@@ -80,7 +81,7 @@ COLORMAP = {k: '\x1b[%sm' % v for k, v in {
 }.items()}
 
 # Disguise as Firefox on Ubuntu
-USER_AGENT = ('Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:53.0) Gecko/20100101 Firefox/53.0')
+USER_AGENT = ('Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:54.0) Gecko/20100101 Firefox/54.0')
 ua = True  # User Agent is enabled by default
 
 # Self-upgrade parameters
@@ -387,6 +388,7 @@ class GoogleUrl(object):
                 site: str
                 start: int
                 tld: str
+                unfilter: bool
 
         Other Parameters
         ----------------
@@ -428,6 +430,8 @@ class GoogleUrl(object):
             self._start = opts['start']
         if 'tld' in opts:
             self._tld = opts['tld']
+        if 'unfilter' in opts and opts['unfilter']:
+            qd['filter'] = 0
 
     def set_queries(self, **kwargs):
         """Forcefully set queries outside the normal `update` mechanism.
@@ -669,12 +673,20 @@ class GoogleConnection(object):
         host_display = host + (':%d' % port if port else '')
 
         proxy = self._proxy
+
         if proxy:
-            logger.debug('Connecting to proxy server %s', proxy)
-            self._conn = TLS1_2Connection(proxy, timeout=timeout)
+            proxy_user_passwd, proxy_host_port = parse_proxy_spec(proxy)
+
+            logger.debug('Connecting to proxy server %s', proxy_host_port)
+            self._conn = TLS1_2Connection(proxy_host_port, timeout=timeout)
 
             logger.debug('Tunnelling to host %s' % host_display)
-            self._conn.set_tunnel(host, port=port)
+            connect_headers = {}
+            if proxy_user_passwd:
+                connect_headers['Proxy-Authorization'] = 'Basic %s' % base64.b64encode(
+                    proxy_user_passwd.encode('utf-8')
+                ).decode('utf-8')
+            self._conn.set_tunnel(host, port=port, headers=connect_headers)
 
             try:
                 self._conn.connect(self._notweak)
@@ -1314,6 +1326,10 @@ class GoogleParser(html.parser.HTMLParser):
             if attrs['href'].startswith('/search'):
                 return
 
+            # Skip card results
+            if not ua and "fl" in self.classes(attrs):
+                return
+
             self.url = attrs['href']
             try:
                 start = self.url.index('?q=') + len('?q=')
@@ -2236,16 +2252,35 @@ def self_upgrade(include_git=False):
 # Miscellaneous functions
 
 def https_proxy_from_environment():
-    if 'https_proxy' not in os.environ:
-        return None
-    proxy = urllib.parse.urlparse(os.environ['https_proxy']).netloc
-    # urlparse recognizes a netloc only if it is introduced by '//'
-    # (https://docs.python.org/3/library/urllib.parse.html#urllib.parse.urlparse).
-    # This means a bare host:port like 'localhost:8118' won't be
-    # recognized. In that case, try again with the '//' prefix.
-    if not proxy:
-        proxy = urllib.parse.urlparse('//' + os.environ['https_proxy']).netloc
-    return proxy if proxy else None
+    return os.getenv('https_proxy')
+
+
+def parse_proxy_spec(proxyspec):
+    if '://' in proxyspec:
+        pos = proxyspec.find('://')
+        scheme = proxyspec[:pos]
+        proxyspec = proxyspec[pos+3:]
+        if scheme.lower() != 'http':
+            # Only support HTTP proxies.
+            #
+            # In particular, we don't support HTTPS proxies since we
+            # only speak plain HTTP to the proxy server, so don't give
+            # users a false sense of security.
+            raise NotImplementedError('Unsupported proxy scheme %s.' % scheme)
+
+    if '@' in proxyspec:
+        pos = proxyspec.find('@')
+        user_passwd = urllib.parse.unquote(proxyspec[:pos])
+        host_port = proxyspec[pos+1:]
+    else:
+        user_passwd = None
+        host_port = proxyspec
+
+    if ':' not in host_port:
+        # Use port 1080 as default, following curl.
+        host_port += ':1080'
+
+    return user_passwd, host_port
 
 
 def parse_args(args=None, namespace=None):
@@ -2293,13 +2328,15 @@ def parse_args(args=None, namespace=None):
            '[h5 (5 hrs), d5 (5 days), w5 (5 weeks), m5 (5 months), y5 (5 years)]')
     addarg('-w', '--site', dest='sites', action='append', metavar='SITE',
            help='search a site using Google')
+    addarg('--unfilter', action='store_true', help='do not omit similar results')
     addarg('-p', '--proxy', default=https_proxy_from_environment(),
-           help='tunnel traffic through an HTTPS proxy (HOST:PORT)')
+           help="""tunnel traffic through an HTTP proxy;
+           PROXY is of the form [http://][user:password@]proxyhost[:port]""")
     addarg('--noua', action='store_true', help='disable user agent')
     addarg('--notweak', action='store_true',
            help='disable TCP optimizations and forced TLS 1.2')
     addarg('--json', action='store_true',
-           help='output in JSON format; implies --exact and --noprompt')
+           help='output in JSON format; implies --noprompt')
     addarg('--show-browser-logs', action='store_true',
            help='do not suppress browser output (stdout and stderr)')
     addarg('--np', '--noprompt', dest='noninteractive', action='store_true',
@@ -2322,10 +2359,6 @@ def main():
     try:
         opts = parse_args()
 
-        # --json implies --exact
-        if opts.json:
-            opts.exact = True
-
         # Set logging level
         if opts.debug:
             logger.setLevel(logging.DEBUG)
